package net.jaekl.cfb.db.driver;

import static net.jaekl.cfb.db.Column.Null.*;
import static net.jaekl.cfb.db.Column.Type.*;
import static org.junit.Assert.*;
import net.jaekl.cfb.db.Column;
import net.jaekl.cfb.db.Table;
import net.jaekl.cfb.db.driver.PostgresqlDriver;

import org.junit.Test;

public class PostgresqlDriverTest {

	@Test
	public void testCreateColumnSql() {
		final Object[][] input = {
			{"FIRSTNAME", VARCHAR, 20, NOT_NULL},
			{"LASTNAME", VARCHAR, 32, NOT_NULL},
			{"COMPANYNAME", VARCHAR, 25, NULL},
			{"AGE", INTEGER, -1, NULL},
			{"GENDER", CHAR, 1, NOT_NULL}
		};
		final String[] expected = {
			"FIRSTNAME VARCHAR(20) NOT NULL",
			"LASTNAME VARCHAR(32) NOT NULL",
			"COMPANYNAME VARCHAR(25) NULL",
			"AGE INTEGER NULL",
			"GENDER CHAR(1) NOT NULL"
		};
		
		PostgresqlDriver driver = new PostgresqlDriver();
		
		for (int idx = 0; idx < input.length; ++idx) {
			Object[] spec = input[idx];
			Column column = Column.construct(spec);
			String actual = driver.createColumnSql(column);
			
			assertEquals(expected[idx], actual);
		}
		
	}

	@Test
	public void testCreateTableSql() {
		Object[][][] input = {
				{
					{"STUDENT"},
					{"ID", INTEGER, -1, NOT_NULL},
					{"NAME", VARCHAR, 64, NOT_NULL}
				}
		};
		
		String[] expected = {
				"CREATE TABLE STUDENT(ID INTEGER NOT NULL, NAME VARCHAR(64) NOT NULL)"
		};
		
		PostgresqlDriver driver = new PostgresqlDriver();
		
		for (int idx = 0; idx < input.length; ++idx) {
			Table table = Table.construct(input[idx]);
			String actual = driver.createTableSql(table);
			assertEquals(expected[idx], actual);
		}
	}

}
